"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mintMetadataServerSideProps = void 0;
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const nodewallet_1 = __importDefault(require("@project-serum/anchor/dist/cjs/nodewallet"));
const Wallet_1 = require("../components/Wallet");
const fungible_entangler_1 = require("@strata-foundation/fungible-entangler");
const client_1 = require("@apollo/client");
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const react_1 = require("@strata-foundation/react");
const mintMetadataServerSideProps = (context) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e;
    const { endpoint } = (0, react_1.getClusterAndEndpoint)((context.query.cluster || Wallet_1.DEFAULT_ENDPOINT));
    const apollo = new client_1.ApolloClient({
        uri: "https://graph.holaplex.com/v1",
        cache: new client_1.InMemoryCache(),
    });
    const connection = new web3_js_1.Connection(endpoint, {});
    const provider = new anchor_1.AnchorProvider(connection, new nodewallet_1.default(web3_js_1.Keypair.generate()), {});
    let mintKeyStr;
    if ((_a = context.params) === null || _a === void 0 ? void 0 : _a.mintKey) {
        mintKeyStr = (_b = context.params) === null || _b === void 0 ? void 0 : _b.mintKey;
    }
    else if ((_c = context.params) === null || _c === void 0 ? void 0 : _c.id) {
        mintKeyStr = (_d = context.params) === null || _d === void 0 ? void 0 : _d.id;
        const id = new web3_js_1.PublicKey(mintKeyStr);
        const fungibleEntanglerSdk = yield fungible_entangler_1.FungibleEntangler.init(provider);
        let childEntangler = yield fungibleEntanglerSdk.getChildEntangler(id);
        if (childEntangler) {
            const parentEntangler = yield fungibleEntanglerSdk.getParentEntangler(childEntangler.parentEntangler);
            mintKeyStr = parentEntangler === null || parentEntangler === void 0 ? void 0 : parentEntangler.parentMint.toString();
        }
    }
    const mint = new web3_js_1.PublicKey(mintKeyStr);
    const address = (yield mpl_token_metadata_1.Metadata.getPDA(mint)).toBase58();
    const result = yield apollo.query({
        query: (0, client_1.gql) `
      query GetUrl($address: String!) {
        nft(address: $address) {
          name
          description
          image
        }
      }
    `,
        variables: {
            address,
        },
    });
    const { name, description, image } = (((_e = result.data) === null || _e === void 0 ? void 0 : _e.nft) || {});
    return {
        props: {
            name: name || null,
            description: description || null,
            image: image || null,
        },
    };
});
exports.mintMetadataServerSideProps = mintMetadataServerSideProps;
//# sourceMappingURL=tokenMetadataServerProps.js.map